﻿Module DirStruct
    ''' <summary>
    ''' سجل المجلد، يحتوي على الدوال الخاصة بالتعامل مع المجلدات
    ''' </summary>
    ''' <remarks></remarks>

    Partial Structure Directory
        ' هذا التعريف الجزئي يحتوي على الدوال الخاصة التي تستخدم داخل السجل فقط
        Private X As Integer
        
        Private Shared Function PathExists(ByVal Path As String) As Boolean
            Try
                GetAttr(Path)
            Catch ex As Exception
                If ex.Message Like "*not found*" Then Return False
            End Try
            Return True
        End Function

        Private Shared Function DelDir(ByVal Path As String) As String
            Try
                SetAttr(Path, FileAttribute.Normal)
                RmDir(Path)
                Return ""
            Catch Ex As Exception
                Return "Error In Dir " & Path & " : " & Ex.Message
            End Try
        End Function

        Private Shared Function DelFile(ByVal Path As String) As String
            Try
                SetAttr(Path, FileAttribute.Normal)
                Kill(Path)
                Return ""
            Catch Ex As Exception
                Return "Error In File " & Path & " : " & Ex.Message
            End Try
        End Function

        Private Shared Function ScanDir(ByVal Path As String, ByVal DirAction As  _
                       Action, ByVal FileAction As Action, Optional ByVal ParentFirst _
                       As Boolean = True) As String

            If Not PathExists(Path) Then Return "Path Not Found"
            Dim FullName As String
            Try
                Dim Contents() As String = GetDirContents(Path)
                If UBound(Contents) = 0 Then Return "Dir Is Empty"
                Dim ErrMsg As String
                For I As Integer = 0 To UBound(Contents)
                    If Path Like "*\" Then
                        FullName = Path & Contents(I)
                    Else
                        FullName = Path & "\" & Contents(I)
                    End If

                    If (GetAttr(FullName) And vbDirectory) = 0 Then
                        ' هذا العنصر ملف
                        If Not FileAction Is Nothing Then
                            ErrMsg = FileAction(FullName)
                            If ErrMsg <> "" Then Return ErrMsg
                        End If
                    Else ' هذا العنصر مجلد
                        If ParentFirst AndAlso Not DirAction Is Nothing Then
                            ErrMsg = DirAction(FullName)
                            If ErrMsg <> "" Then Return ErrMsg
                        End If
                        ErrMsg = ScanDir(FullName, DirAction, FileAction, ParentFirst)
                        If ErrMsg <> "" Then Return ErrMsg

                        If ParentFirst = False AndAlso _
                                Not DirAction Is Nothing Then
                            ErrMsg = DirAction(FullName)
                            If ErrMsg <> "" Then Return ErrMsg
                        End If
                    End If

                Next
            Catch Ex As Exception
                Return "Error In Path " & Path & " : " & Ex.Message
            End Try
            Return ""
        End Function

        Private Shared Function DirLocation(ByVal Path As String, ByVal NewPath As _
                 String, ByVal DirAction As Action2, ByVal FileAction As  _
                Action2, Optional ByVal ParentFirst As Boolean = True, _
                Optional ByVal MoreDirAction As Action = Nothing) As String

            Dim FullName, FullName2 As String
            Try
                Dim Contents() As String = GetDirContents(Path)
                Dim ErrMsg As String
                For I As Integer = 0 To UBound(Contents)
                    If Path Like "*\" Then
                        FullName = Path & Contents(I)
                    Else
                        FullName = Path & "\" & Contents(I)
                    End If

                    If NewPath Like "*\" Then
                        FullName2 = NewPath & Contents(I)
                    Else
                        FullName2 = NewPath & "\" & Contents(I)
                    End If

                    If (GetAttr(FullName) And vbDirectory) = 0 Then
                        ' هذا العنصر ملف
                        If Not FileAction Is Nothing Then
                            ErrMsg = FileAction(FullName, FullName2)
                            If ErrMsg <> "" Then Return ErrMsg
                        End If
                    Else ' هذا العنصر مجلد
                        If ParentFirst AndAlso Not DirAction _
                                 Is Nothing Then
                            ErrMsg = DirAction(FullName, FullName2)
                            If ErrMsg <> "" Then Return ErrMsg
                        End If

                        ErrMsg = DirLocation(FullName, FullName2, _
                                          DirAction, FileAction, ParentFirst)
                        If ErrMsg <> "" Then Return ErrMsg

                        If ParentFirst = False AndAlso Not DirAction _
                                   Is Nothing Then
                            ErrMsg = DirLocation(FullName, FullName2, DirAction, FileAction, ParentFirst, MoreDirAction)
                            If ErrMsg <> "" Then Return ErrMsg
                        End If
                        ' تعديل هام
                        If Not MoreDirAction Is Nothing Then
                            ErrMsg = MoreDirAction(FullName)
                            If ErrMsg <> "" Then Return ErrMsg
                        End If
                    End If
                Next
            Catch Ex As Exception
                Return "Error In Path " & Path & " : " & Ex.Message
            End Try
            Return ""
        End Function

        Private Shared Function GetDirName(ByVal Path As String) As String
            Dim St As Integer = Len(Path) - 1
            Dim Pos As String = InStrRev(Path, "\", St)
            If Pos = 0 Then ' هذا مسار محرك
                Return ""
            Else
                Return Mid(Path, Pos + 1, St - Pos)
            End If
        End Function

        Private Shared Function CheckPaths(ByRef Path1 As String, ByRef Path2 As String) As String
            If Not PathExists(Path1) Then Return "Source Dir Does Not Exist"
            Path1 = Replace(Path1, "/", "\")
            If Not Path1 Like "*\" Then Path1 += "\"
            Path2 = Replace(Path2, "/", "\")
            If Not Path2 Like "*\" Then Path2 += "\"
            If LCase(Path1) = LCase(Path2) Then Return "Cant move dir to the same location"
            If LCase(Path2) Like LCase(Path1) + "*" Then Return "Cant move dir inside itself"
            Return ""
        End Function

        Private Shared Function CopyDir2(ByVal Path As String, ByVal NewPath As String) As String
            Try
                If Not PathExists(NewPath) Then MkDir(NewPath)
                Return ""
            Catch Ex As Exception
                Return "Error In Dir " & Path & " : " & Ex.Message
            End Try
        End Function

        Private Shared Function CopyFile(ByVal Path As String, ByVal NewPath As String) As String
            Try
                If PathExists(NewPath) Then
                    Dim ErrMsg As String = DelFile(NewPath)
                    If ErrMsg <> "" Then Return ErrMsg
                End If
                FileCopy(Path, NewPath)
                Return ""
            Catch Ex As Exception
                Return "Error In File " & Path & " : " & Ex.Message
            End Try
        End Function

        Private Shared Function MoveFile(ByVal Path As String, ByVal NewPath As String) _
                 As String
            Try
                If PathExists(NewPath) Then
                    Dim ErrMsg As String = DelFile(NewPath)
                    If ErrMsg <> "" Then Return ErrMsg
                End If
                Rename(Path, NewPath)
                Return ""
            Catch Ex As Exception
                Return "Error In File " & Path & " : " & Ex.Message
            End Try
        End Function

        Private Delegate Function Action(ByVal Path As String) As String
        Private Delegate Function Action2(ByVal Path As String, ByVal NewPath As String) As String
    End Structure

    ' هذا التعريف الجزئي يحتوي على الأعضاء العامة التي يمكن استدعاؤها من خارج السجل
    Partial Structure Directory

        ' الأعضاء العامة التي يمكن استخدامها من خارج السجل
        Shared Function GetDirs(ByVal Path As String, Optional ByVal Attr As FileAttribute = -1) As String()
            Path = Replace(Path, "/", "\")
            If Not Path Like "*\" Then Path += "\"

            If Attr = -1 Then ' لم يرسل المستخدم المعامل الثاني 
                ' السمة الوحيدة التي نريدها هي أن يكون هذا العنصر مجلدا
                Attr = vbDirectory
            Else ' سنضيف سمة المجلد إلى السمات المطلوبة
                Attr = Attr Or vbDirectory
            End If

            Dim L As Integer = 255
            Dim Dirs(L) As String
            Dim C As Integer = -1

            Dim DirName As String = Dir(Path, vbDirectory)
            While DirName <> ""
                ' يجب تكوين المسار الكامل
                DirName = Path + DirName
                If (GetAttr(DirName) And Attr) = Attr Then
                    C += 1
                    If C > L Then
                        L += 255
                        ReDim Preserve Dirs(L)
                    End If
                    Dirs(C) = DirName
                End If
                DirName = Dir()
            End While
            ReDim Preserve Dirs(C)
            Return Dirs
        End Function

        Shared Function GetFiles(ByVal Path As String, _
               Optional ByVal Attr As FileAttribute = 47) As String()
            Path = Replace(Path, "/", "\")
            If Not Path Like "*\" Then Path += "\"
            Attr = Attr And 47
            Dim L As Integer = 255
            Dim Files(L) As String
            Dim C As Integer = -1

            Dim FileName As String = Dir(Path, Attr)
            While FileName <> ""
                FileName = Path + FileName
                If (GetAttr(FileName) And vbDirectory) = 0 Then
                    C += 1
                    If C > L Then
                        L += 255
                        ReDim Preserve Files(L)
                    End If
                    Files(C) = FileName
                End If
                FileName = Dir()
            End While
            ReDim Preserve Files(C)
            Return Files
        End Function

        Shared Function GetDirContents(ByVal Path As String, Optional ByVal Attr As FileAttribute = 63) As String()
            Path = Replace(Path, "/", "\")
            If Not Path Like "*\" Then Path += "\"

            ' سنضيف سمة المجلد إلى السمات المطلوبة
            Attr = Attr Or vbDirectory

            Dim L As Integer = 255
            Dim Dirs(L) As String
            Dim C As Integer = -1

            Dim ContentName As String = Dir(Path, Attr)
            While ContentName <> ""
                C += 1
                If C > L Then
                    L += 255
                    ReDim Preserve Dirs(L)
                End If
                Dirs(C) = ContentName
                ContentName = Dir()
            End While
            ReDim Preserve Dirs(C)
            Return Dirs
        End Function

        Shared Function DeleteDir(ByVal Path As String) As String
            'حذف محاويات المجلد 
            Dim X As String = ScanDir(Path, AddressOf DelDir, AddressOf DelFile, False)
            If X <> "" Then Return ("Error: " & X)
            ' حذف المجلد نفسه
            Return DelDir(Path)
        End Function

        Shared Function CopyDir(ByVal Path As String, ByVal NewPath As String) As String
            Try
                Dim ErrMsg As String = CheckPaths(Path, NewPath)
                If ErrMsg <> "" Then Return ErrMsg
                Dim DirName As String = NewPath + GetDirName(Path)
                ' إنشاء المجلد على المسار الجديد
                If Not PathExists(DirName) Then MkDir(DirName)
                ' نسخ محتويات المجلد
                Return DirLocation(Path, DirName, AddressOf CopyDir2, AddressOf CopyFile)
            Catch Ex As Exception
                Return "Error In Dir " & Path & " : " & Ex.Message
            End Try
        End Function

        Shared Function MoveDir(ByVal Path As String, ByVal NewPath As String) As String
            Try
                Dim ErrMsg As String = CheckPaths(Path, NewPath)
                If ErrMsg <> "" Then Return ErrMsg
                Dim DirName As String = NewPath + GetDirName(Path)

                If LCase(Mid(Path, 1, 1)) = _
                             LCase(Mid(NewPath, 1, 1)) Then
                    ' المساران على نفس المحرك لأنهما يبدآن بنفس الحرف
                    Rename(Path, DirName)
                    Return "" ' انتهت العملية، ولا حاجة لتنفيذ باقي الإجراء
                End If

                ' إنشاء المجلد على المسار الجديد
                If Not PathExists(DirName) Then MkDir(DirName)
                ' نقل محتويات المجلد 
                ErrMsg = DirLocation(Path, DirName, AddressOf _
                      CopyDir2, AddressOf MoveFile, True, _
                      AddressOf DelDir)
                If ErrMsg <> "" Then Return ErrMsg
                ' حذف المجلد نفسه
                Return DelDir(Path)
            Catch Ex As Exception
                Return "Error In Dir " & Path & " : " & Ex.Message
            End Try
        End Function

        Shared Function GetDirSize(ByVal Path As String) As Long
            Dim FullName As String, DirSize As Long = 0
            Try
                Dim Contents() As String = GetDirContents(Path)
                For I As Integer = 0 To UBound(Contents)
                    If Path Like "*\" Then
                        FullName = Path & Contents(I)
                    Else
                        FullName = Path & "\" & Contents(I)
                    End If
                    If (GetAttr(FullName) And vbDirectory) = 0 Then
                        ' هذا العنصر ملف.. سنضيف حجمه إلى حجم المجلد
                        DirSize += FileLen(FullName)
                    Else ' هذا العنصر مجلد
                        ' سنضيف حجمه إلى حجم المجلد باستدعاء الدالة لنفسها
                        DirSize += GetDirSize(FullName)
                    End If
                Next
            Catch Ex As Exception
                Return -1
            End Try
            Return DirSize
        End Function

        Shared Function Exists(ByVal Path As String) As Boolean
            Return PathExists(Path)
        End Function

        Shared Function CreateDir(ByVal Path As String) As String
            Try
                If Not PathExists(Path) Then MkDir(Path)
            Catch Ex As Exception
                Return "Error In Dir " & Path & " : " & Ex.Message
            End Try
            Return ""
        End Function

        Shared Property CurrentDir() As String
            Get
                Return CurDir()
            End Get
            Set(ByVal Value As String)
                ChDrive(Value)
                ChDir(Value)
            End Set
        End Property

        Shared Property Attributes(ByVal Path As String) _
                       As FileAttribute
            Get
                Return GetAttr(Path)
            End Get
            Set(ByVal value As FileAttribute)
                SetAttr(Path, value)
            End Set
        End Property
    End Structure

    ' هذا التعريف الجزئي يحتوي على دوال خاصة بالمحركات فقط
    Partial Structure Directory
        Function GetDrives() As Char()
            Dim Drives(23) As Char
            Dim C As Integer = -1

            For I As Integer = Asc("C"c) To Asc("Z"c)
                Try
                    GetAttr(Chr(I) + ":\")
                    C += 1
                    Drives(C) = Chr(I)
                Catch

                End Try
            Next
            ReDim Preserve Drives(C)
            Return Drives
        End Function

        Shared Property CurrentDrive() As String
            ' لاحظ أن المحرّك الحالي هو أول ثلاثة حروف في المجلد الحالي (اسم المجلد والعلامتين :\)
            Get
                Return Mid(CurDir(), 1, 3)
            End Get
            Set(ByVal Value As String)
                ChDrive(Value)
            End Set
        End Property

    End Structure

End Module
