﻿Module PathStruct
    Structure Path
        Private X As Integer

        Private Shared Sub FixPath(ByRef Path As String)
            Path = Replace(Path, "/", "\")
            If Path Like "*\" Then Path = Mid(Path, 1, _
                 Len(Path) - 1)
        End Sub

        Shared Function GetExtension(ByVal Path As String) _
                   As String
            FixPath(Path)
            Dim I As Integer = InStrRev(Path, ".")
            If I = 0 Then Return ""
            Return Mid(Path, I + 1, Len(Path) - I)
        End Function

        Shared Function GetFileName(ByVal Path As String) As String
            FixPath(Path)
            Dim I As Integer = InStrRev(Path, "\")
            If I = 0 Then Return ""
            Return Mid(Path, I + 1, Len(Path) - I)
        End Function
        Shared Function GetFileNameWithoutEx(ByVal Path As _
                             String) As String
            FixPath(Path)
            Dim I As Integer = InStrRev(Path, ".")

            If I = 0 Then
                Return GetFileName(Path)
            Else
                Dim J As Integer = InStrRev(Path, "\")
                If J = 0 Then Return ""
                Return Mid(Path, J + 1, I - J - 1)
            End If
        End Function

        Shared Function GetDirName(ByVal Path As String) As String
            FixPath(Path)
            Dim I As Integer = InStrRev(Path, ".")
            If I = 0 Then
                Return GetFileName(Path)
            Else
                Dim J As Integer = InStrRev(Path, "\")
                If J = 0 Then Return ""
                Dim K As Integer = InStrRev(Path, "\", J - 1)
                If J = 0 Then Return ""
                Return Mid(Path, K + 1, J - K - 1)
            End If
        End Function

        Shared Function GetParentPath(ByVal Path As String) _
                        As String
            FixPath(Path)
            Dim I As Integer = InStrRev(Path, "\")
            If I = 0 Then Return ""
            Return Mid(Path, 1, I - 1)
        End Function

        Shared Function GetRootPath(ByVal Path As String) As String
            FixPath(Path)
            Dim I As Integer = InStr(Path, "\")
            If I = 0 Then Return ""
            Return Mid(Path, 1, I)
        End Function

        Shared Function ChangeExtension(ByVal Path As String, ByVal Ext As String) As String
            FixPath(Path)
            Dim I As Integer = InStrRev(Path, ".")
            If I = 0 Then Return Path + "." + Ext
            Return Mid(Path, 1, I) + Ext
        End Function

        Shared Function Combine(ByVal Path1 As String, ByVal _
                        Path2 As String) As String
            FixPath(Path1)
            FixPath(Path2)
            If Path2 Like "*:*" Then Return ""
            Return Path1 + "\" + Path2
        End Function


    End Structure

End Module
