﻿Imports VB = Microsoft.VisualBasic

Public Class FileStream : Inherits Stream

    Protected No As Integer = FreeFile()
    Dim pFileName As String

    Function FileExists(ByVal FileName As String) As Boolean
        Try
            Dim L As Long = FileLen(FileName)
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Enum FileMode
        Append
        Create
        CreateNew
        Open
        OpenOrCreate
        Truncate
    End Enum

    Private Sub CheckFile(ByVal mode As FileMode)
        Dim Exists As Boolean = FileExists(pFileName)
        Select Case mode
            Case FileMode.CreateNew
                If Exists Then
                    Throw New Exception("الملف موجود بالفعل")
                End If
            Case FileMode.Create, FileMode.Truncate
                ' محو محتويات الملف بمحو الملف نفسه
                Kill(pFileName)
            Case FileMode.Open
                If Not Exists Then
                    Throw New Exception("الملف غير موجود")
                End If
        End Select
    End Sub

    Public Sub New(ByVal FileName As String, Optional _
                ByVal mode As FileMode = FileMode.OpenOrCreate, _
                Optional ByVal access As OpenAccess = _
                OpenAccess.ReadWrite, Optional ByVal share As  _
                OpenShare = OpenShare.LockWrite)
        pFileName = FileName
        CheckFile(mode)
        FileOpen(No, FileName, OpenMode.Binary, access, share)
        If mode = FileMode.Append Then
            VB.Seek(No, LOF(No) + 1)
        End If
        pCanWrite = Not (access = OpenAccess.Read)
        pCanRead = Not (access = OpenAccess.Write)
    End Sub

    Public Overrides Sub Close()
        FileClose(No)
    End Sub

    Public Overrides Function ReadByte() As Integer
        If pCanRead = False Then
            Throw New Exception("هذا الملف مفتوح للكتابة فقط")
        End If
        If EOF(No) Then Return -1
        Dim B As Byte
        FileGet(No, B) : Return B
    End Function

    Public Overrides Function Read(ByRef Data() As Byte, _
                Optional ByVal St As Integer = 0, Optional ByVal _
                Count As Integer = 1) As Integer
        If pCanRead = False Then
            Throw New Exception("هذا الملف مفتوح للكتابة فقط")
        End If
        If VB.Seek(No) - 1 + Count > LOF(No) Then Return -1
        Dim B As Byte
        For I As Integer = 0 To Count - 1
            FileGet(No, B)
            Data(St + I) = B
        Next
        Return 0
    End Function

    Public Overrides Sub WriteByte(ByVal value As Byte)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        FilePut(No, value)
    End Sub

    Public Overrides Sub Write(ByVal Data() As Byte, _
                Optional ByVal St As Integer = 0, Optional ByVal Count As Integer = 1)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        For I As Integer = St To St + Count - 1
            FilePut(No, Data(I))
        Next
    End Sub

    Public Overrides Function Seek(ByVal Offset As Long, _
                Optional ByVal origin As SeekOrigin = SeekOrigin.Current) As Long
        Dim Pos As Long
        Select Case origin
            Case SeekOrigin.Begin
                Pos = Offset
            Case SeekOrigin.Current
                Pos = VB.Seek(No) + Offset - 1
            Case Else 'SeekOrigin.End 
                Pos = LOF(No) - 1 + Offset
        End Select
        If Pos >= LOF(No) OrElse Pos < 0 Then
            Throw New Exception("لا يمكن الانتقال إلى هذا الموضع")
        End If
        VB.Seek(No, Pos + 1)
        Return Pos
    End Function

    Public Overrides ReadOnly Property CanRead() As Boolean
        Get
            Return pCanRead
        End Get
    End Property

    Public Overrides ReadOnly Property CanWrite() As Boolean
        Get
            Return pCanWrite
        End Get
    End Property

    Public Overrides ReadOnly Property Length() As Long
        Get
            Return LOF(No)
        End Get
    End Property

    Public Overrides Property Position() As Long
        Get
            Return VB.Seek(No) - 1
        End Get
        Set(ByVal Pos As Long)
            If Pos >= LOF(No) OrElse Pos < 0 Then
                Throw New Exception("لا يمكن الانتقال إلى هذا الموضع")
            End If
            VB.Seek(No, Pos + 1)
        End Set
    End Property

    ReadOnly Property FileName() As String
        Get
            Return pFileName
        End Get
    End Property

    Public Sub WriteShort(ByVal value As Short)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        FilePut(No, value)
    End Sub

    Public Sub WriteInteger(ByVal value As Integer)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        FilePut(No, value)
    End Sub

    Public Sub WriteString(ByVal value As String)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        FilePut(No, value)
    End Sub

    Public Function ReadShort() As Short
        If pCanRead = False Then
            Throw New Exception("هذا الملف مفتوح للكتابة فقط")
        End If
        If EOF(No) Then Throw New Exception("لقد تجاوزت نهاية الملف")
        Dim S As Short
        FileGet(No, S)
        Return S
    End Function

    Public Function ReadInteger() As Integer
        If pCanRead = False Then
            Throw New Exception("هذا الملف مفتوح للكتابة فقط")
        End If
        If EOF(No) Then Throw New Exception("لقد تجاوزت نهاية الملف")
        Dim I As Integer
        FileGet(No, I)
        Return I
    End Function

    Public Function ReadString(ByVal ChrCount As Integer) _
                As String
        If pCanRead = False Then
            Throw New Exception("هذا الملف مفتوح للكتابة فقط")
        End If
        If EOF(No) Then Throw New Exception("لقد تجاوزت نهاية الملف")
        If Seek(No) + ChrCount > LOF(No) Then
            ChrCount = LOF(No) - Seek(No)
        End If
        Return InputString(No, ChrCount)
    End Function

    Public Sub WriteStruc(Of StrucType As Structure) _
            (ByVal value As StrucType)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        FilePut(No, value)
    End Sub


    Public Function ReadStruc(Of StrucType As Structure)() _
               As StrucType
        If pCanRead = False Then
            Throw New Exception("هذا الملف مفتوح للكتابة فقط")
        End If
        If EOF(No) Then Return Nothing
        Dim Rec As StrucType
        FileGet(No, Rec)
        Return Rec
    End Function

    Public Function ToArray() As Byte()
        Dim Pos As Long = VB.Seek(No)
        VB.Seek(No, 1)
        Dim A(LOF(No) - 1) As Byte
        FileGet(No, A)
        VB.Seek(No, Pos)
        Return A
    End Function

    Public Overridable Sub WriteTo(ByVal S As Stream, Optional _
        ByVal St As Integer = 0, Optional ByVal En As Integer = -1)
        If En = -1 Then En = LOF(No) - 1
        Dim Pos As Long = VB.Seek(No)
        VB.Seek(No, St + 1)
        Dim B As Byte
        For I As Integer = 0 To En - St
            FileGet(No, B)
            S.WriteByte(B)
        Next
        VB.Seek(No, Pos)
    End Sub

    Public Sub Lock(ByVal position As Long, ByVal length As Long)
        VB.Lock(No, position + 1, position + length)
    End Sub

    Public Sub Unlock(ByVal position As Long, ByVal length As Long)
        VB.Unlock(No, position + 1, position + length)
    End Sub

    Function GetFileOpenAttr() As OpenMode
        Return FileAttr(No)
    End Function

End Class
