﻿Public Class MemoryRecord(Of RType As Structure)
    Inherits Record(Of RType)

    Dim M() As RType, pCapacity As Long = 255
    Dim pLength, pPosition As Long

    Public Overridable Property Capacity() As Long
        Get
            Return pCapacity
        End Get
        Set(ByVal value As Long)
            pCapacity = value
            ReDim Preserve M(pCapacity - 1)
        End Set
    End Property
    
    Sub New()
        ReDim M(pCapacity - 1) : pLength = 0 : pPosition = 0
    End Sub

    Sub New(ByVal Capacity As Long)
        pCapacity = Capacity : ReDim M(pCapacity - 1)
        pLength = 0 : pPosition = 0
    End Sub

    Sub New(ByVal Data() As RType, ByVal CanWrite As _
            Boolean, Optional ByVal St As Long = 0, Optional ByVal En As Long = -1)
        pCanWrite = CanWrite
        If En = -1 Then En = UBound(Data)
        pLength = En - St + 1
        If pCanWrite Then
            If pLength > pCapacity Then
                pCapacity += 255 : ReDim Preserve M(pCapacity - 1)
            End If
        Else : pCapacity = pLength
        End If
        ReDim M(pCapacity - 1)
        For I As Integer = St To En
            M(I) = Data(I)
        Next
        pPosition = 0
    End Sub

    Public Overrides Sub Close()
        Erase M
    End Sub

    Public Overrides Function ReadRecord(Optional ByVal _
                RecNo As Long = -1) As RType
        If RecNo > -1 Then pPosition = RecNo
        If pPosition >= pLength Then Return Nothing
        ReadRecord = M(pPosition)
        pPosition += 1
    End Function

    Public Overrides Sub Read(ByRef Rec() As RType, _
           Optional ByVal RecNo As Long = -1, Optional ByVal _
           St As Integer = 0, Optional ByVal Count As Integer = 1)
        If RecNo > -1 Then pPosition = RecNo
        If pPosition + Count - 1 >= pLength Then
            Count = pLength - pPosition
            If Count < 1 Then Return
        End If

        For I As Integer = 0 To Count - 1
            Rec(St + I) = M(pPosition)
            pPosition += 1
        Next
    End Sub

    Public Overrides Sub WriteRecord(ByVal Value As RType, _
                Optional ByVal RecNo As Long = -1)
        If pCanWrite = False Then
            Throw New Exception("هذه البيانات للقراءة فقط")
        End If
        If RecNo > -1 Then pPosition = RecNo
        If pPosition >= pLength Then
            pLength = pPosition + 1
            If pLength > pCapacity Then
                pCapacity += 255
                ReDim Preserve M(pCapacity - 1)
            End If
        End If
        M(pPosition) = Value
        pPosition += 1
    End Sub

    Public Overrides Sub Write(ByVal Data() As RType, _
                Optional ByVal RecNo As Long = -1, _
                Optional ByVal St As Integer = 0, _
                Optional ByVal Count As Integer = 1)
        If pCanWrite = False Then
            Throw New Exception("هذه البيانات للقراءة فقط")
        End If
        If RecNo > -1 Then pPosition = RecNo
        If pPosition + Count - 1 >= pLength Then
            pLength = pPosition + Count
            If pLength > pCapacity Then
                pCapacity += 255
                ReDim Preserve M(pCapacity - 1)
            End If
        End If

        For I As Integer = St To St + Count - 1
            M(pPosition) = Data(I)
            pPosition += 1
        Next
    End Sub

    Public Overrides ReadOnly Property CanRead() As Boolean
        Get
            Return pCanRead
        End Get
    End Property

    Public Overrides ReadOnly Property CanWrite() As Boolean
        Get
            Return pCanWrite
        End Get
    End Property

    Public Overrides ReadOnly Property Length() As Long
        Get
            Return pLength
        End Get
    End Property

    Public Overrides Property Position() As Long
        Get
            Return pPosition
        End Get
        Set(ByVal Pos As Long)
            If Pos > pLength OrElse Pos < 0 Then
                Throw New Exception("لا يمكن الانتقال إلى هذا الموضع")
            End If
            pPosition = Pos
        End Set
    End Property

    Public Function GetBuffer() As RType()
        Return M
    End Function

    Public Function ToArray() As RType()
        Dim A(pLength - 1) As RType
        For I As Integer = 0 To pLength - 1
            A(I) = M(I)
        Next
        Return A
    End Function

    Public Overridable Sub WriteTo(ByVal R As Record(Of RType), Optional ByVal St As Integer = 0, Optional ByVal En As Integer = -1)
        If En = -1 Then En = pLength - 1
        R.Write(M, St, En - St + 1)
    End Sub
End Class
