﻿Imports Vb = Microsoft.VisualBasic

Interface IEncoder
    Property Key1() As Byte
    Property Key2() As Byte
    Function Endode(ByVal B As Byte) As Byte
    Function Dedode(ByVal B As Byte) As Byte
End Interface

Interface IContent
    Function GetBytes() As Byte()
    Sub LoadBytes(ByVal B() As Byte)
End Interface

Class EncodedFile : Inherits FileStream
    Dim Enc As IEncoder
    Public Sub New(ByVal FileName As String, ByVal Encoder _
                As IEncoder, Optional ByVal mode As FileMode = _
                FileMode.OpenOrCreate, Optional ByVal access As  _
                OpenAccess = OpenAccess.ReadWrite, Optional _
                ByVal share As OpenShare = OpenShare.LockWrite)
        MyBase.New(FileName, mode, access, share)
        Enc = Encoder
    End Sub

    Public Overrides Sub WriteByte(ByVal value As Byte)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        FilePut(No, Enc.Endode(value))
    End Sub

    Public Overrides Function ReadByte() As Integer
        If pCanRead = False Then
            Throw New Exception("هذا الملف مفتوح للكتابة فقط")
        End If
        If EOF(No) Then Return -1
        Dim B As Byte : FileGet(No, B)
        Return Enc.Dedode(B)
    End Function

    Public Overrides Sub Write(ByVal Data() As Byte, Optional ByVal St As Integer = 0, Optional ByVal count As Integer = 1)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        For I As Integer = St To St + count - 1
            FilePut(No, Enc.Endode(Data(I)))
        Next
    End Sub

    Public Overrides Function Read(ByRef Data() As Byte, Optional ByVal St As Integer = 0, Optional ByVal count As Integer = 1) As Integer
        If pCanRead = False Then
            Throw New Exception("هذا الملف مفتوح للكتابة فقط")
        End If
        If VB.Seek(No) - 1 + count > LOF(No) Then Return -1
        Dim B As Byte
        For I As Integer = 0 To count - 1
            FileGet(No, B)
            Data(St + I) = Enc.Dedode(B)
        Next
        Return 0
    End Function

    Public Sub WriteBytes(ByVal Data() As Byte)
        Write(Data, 0, UBound(Data) + 1)
    End Sub

    Public Function ReadBytes(ByRef Data() As Byte) As Integer
        Return Read(Data, 0, UBound(Data) + 1)
    End Function

    Public Shadows Sub WriteInteger(ByVal value As Integer)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        WriteBytes(BitConverter.GetBytes(value))
    End Sub
    
    Public Shadows Function ReadInteger() As Integer
        If pCanRead = False Then
            Throw New Exception("هذا الملف مقتوح للكتابة فقط")
        End If
        If EOF(No) Then Throw New Exception("لقد تجاوزت نهاية الملف")
        Dim B(3) As Byte
        ReadBytes(B) : Return BitConverter.ToInt32(B, 0)
    End Function

    Public Shadows Sub WriteShort(ByVal value As Short)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        WriteBytes(BitConverter.GetBytes(value))
    End Sub

    Public Shadows Function ReadShort() As Short
        If pCanRead = False Then
            Throw New Exception("هذا الملف مقتوح للكتابة فقط")
        End If
        If EOF(No) Then Throw New Exception("لقد تجاوزت نهاية الملف")
        Dim B(1) As Byte
        ReadBytes(B)
        Return BitConverter.ToInt16(B, 0)
    End Function

    Public Shadows Sub WriteString(ByVal value As String)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        Dim B() As Byte
        B = System.Text.Encoding.Default.GetBytes(value)
        Write(B, 0, Ubound(B) + 1)
    End Sub

    Public Shadows Function ReadString(ByVal ChrCount As Integer) As String
        If pCanRead = False Then
            Throw New Exception("هذا الملف مقتوح للكتابة فقط")
        End If
        If EOF(No) Then Throw New Exception("لقد تجاوزت نهاية الملف")
        If Vb.Seek(No) - 1 + ChrCount > LOF(No) Then
            ChrCount = LOF(No) - Vb.Seek(No) + 1
        End If
        Dim B(ChrCount - 1) As Byte
        ReadBytes(B)
        Return System.Text.Encoding.Default.GetString(B)
    End Function

    Public Shadows Sub WriteStruc(Of StrucType As {Structure, IContent})(ByVal Rec As StrucType)
        If pCanWrite = False Then
            Throw New Exception("هذا الملف مفتوح للقراءة فقط")
        End If
        WriteBytes(Rec.GetBytes)
    End Sub

    Public Shadows Function ReadStruc(Of StrucType As {Structure, IContent})() As StrucType
        If pCanRead = False Then
            Throw New Exception("هذا الملف مقتوح للكتابة فقط")
        End If
        If EOF(No) Then Return Nothing
        Dim Rec As StrucType, B(Len(Rec) - 1) As Byte
        ReadBytes(B)
        Rec.LoadBytes(B)
        Return Rec
    End Function

End Class
